/**
 * Definition des macros de gestion d'erreur, on peut definit NO_FILELOCATION, NO_DEBUG et NO_VERBOSE
 * @author Thierry JOUIN
 * @version 1.1
 * @date 19/10/2001
 */

#ifndef _CERROR_HH_
#define _CERROR_HH_

#include <string>
#include <iostream>
#include <strstream>
#include <stdio.h>
#include <exception>
#include <stdexcept>

/// Definition de l'iostream de sortie des messages de debug
#define ERROR_IOSTREAM std::cerr
/// Definition de l'iostream de sortie des messages verbeux
#define VERBOSE_IOSTREAM std::cout

/*********************************/
#ifdef NO_FILELOCATION

/// Macro pour afficher l'endroit courant dans le fichier
#define FILELOCATION ""

#else

/// Macro pour afficher l'endroit courant dans le fichier
#define FILELOCATION __FILE__ << ":" << __LINE__ 

#endif

/*********************************/
#ifdef NO_DEBUG

/// Macro pour l'affichage du contenu d'une variable pour le debuggage
#define PRINTVAR( var ) 
/// Affichage d'un warning
#define WARNING( warn ) 

#else

/// Affichage du contenu d'une variable pour le debuggage
#define PRINTVAR( var ) VERBOSE_IOSTREAM << "Var: [" << #var << "] = " << var << std::endl
/// Affichage d'un warning
#define WARNING( warn ) VERBOSE_IOSTREAM << "Wrn: " << FILELOCATION << ": " << warn << std::endl

#endif

#ifndef NO_ASSERT_DEBUG

/// Macro permettant l'envoie d'un type d'exception
#define DERROR(except)
/// Macro permettant l'envoie d'un type d'exception, enrichit d'un message
#define DERRORMESSAGE(except,message) 
/// Macro permettant le test d'une expression, si le resultat est faux, une exception est levee
#define DASSERT(expr,except) 
/// Macro permettant le test d'une expression, si le resultat est faux, une exception est levee avec un message
#define DASSERTMESSAGE(expr,except,message)

#else

/// Macro permettant l'envoie d'un type d'exception
#define DERROR(except) { \
char __charBuff[256];\
std::ostrstream __charStream(__charBuff,256);\
__charStream << FILELOCATION << std::ends;\
throw except(__charStream.str()); \
}

/// Macro permettant l'envoie d'un type d'exception, enrichit d'un message
#define DERRORMESSAGE(except,message) { \
char __charBuff[256];\
std::ostrstream __charStream(__charBuff,256);\
__charStream << FILELOCATION << ": " << message << std::ends;\
throw except(__charStream.str()); \
}

/// Macro permettant le test d'une expression, si le resultat est faux, une exception est levee
#define DASSERT(expr,except) { \
if (!(expr)) \
 ERROR(except); \
}

/// Macro permettant le test d'une expression, si le resultat est faux, une exception est levee avec un message
#define DASSERTMESSAGE(expr,except,message){ \
if (!(expr)) \
 ERRORMESSAGE(except,message); \
}

#endif

/*********************************/
#ifdef NO_VERBOSE

/// Affichage d'un message quelconque
#define MSG( msg ) 

#else

/// Affichage d'un message quelconque
#define MSG( msg ) VERBOSE_IOSTREAM << "Msg: " << msg << std::endl

#endif

/*********************************/
/// Affichage d'un message d'erreur
#define PRINTERROR(except) ERROR_IOSTREAM << "Exp: " << except.what() << "(" << FILELOCATION << ")" << std::endl

/// Macro permettant l'envoie d'un type d'exception
#define ERROR(except) { \
char __charBuff[256];\
std::ostrstream __charStream(__charBuff,256);\
__charStream << FILELOCATION << std::ends;\
throw except(__charStream.str()); \
}

/// Macro permettant l'envoie d'un type d'exception, enrichit d'un message
#define ERRORMESSAGE(except,message) { \
char __charBuff[256];\
std::ostrstream __charStream(__charBuff,256);\
__charStream << FILELOCATION << ": " << message << std::ends;\
throw except(__charStream.str()); \
}

/// Macro permettant le test d'une expression, si le resultat est faux, une exception est levee
#define ASSERT(expr,except) { \
if (!(expr)) \
 ERROR(except); \
}

/// Macro permettant le test d'une expression, si le resultat est faux, une exception est levee avec un message
#define ASSERTMESSAGE(expr,except,message){ \
if (!(expr)) \
 ERRORMESSAGE(except,message); \
}

#endif
